@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2000-2003, Ximian, Inc.
  */
@@ -140,6 +140,7 @@ void                soup_message_set_reason_phrase       (SoupMessage        *ms
 void                soup_message_set_method              (SoupMessage        *msg,
                                                           const char         *method);
 
-gboolean            soup_message_is_options_ping         (SoupMessage        *msg);
+void                soup_message_set_http_version        (SoupMessage       *msg,
+						          SoupHTTPVersion    version);
 
 #endif /* __SOUP_MESSAGE_PRIVATE_H__ */
