@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  * soup-message.c: HTTP request/response
  *
@@ -91,7 +91,7 @@ typedef struct {
 	SoupMessagePriority priority;
 
 	gboolean is_top_level_navigation;
-        gboolean options_ping;
+        gboolean is_options_ping;
 } SoupMessagePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (SoupMessage, soup_message, G_TYPE_OBJECT)
@@ -137,7 +137,7 @@ enum {
 	PROP_PRIORITY,
 	PROP_SITE_FOR_COOKIES,
 	PROP_IS_TOP_LEVEL_NAVIGATION,
-        PROP_OPTIONS_PING,
+        PROP_IS_OPTIONS_PING,
 
 	LAST_PROP
 };
@@ -175,8 +175,8 @@ soup_message_finalize (GObject *object)
 
 	g_clear_object (&priv->tls_certificate);
 
-	soup_message_headers_free (priv->request_headers);
-	soup_message_headers_free (priv->response_headers);
+	soup_message_headers_unref (priv->request_headers);
+	soup_message_headers_unref (priv->response_headers);
 	g_clear_object (&priv->request_body_stream);
 
 	g_free (priv->reason_phrase);
@@ -189,11 +189,10 @@ soup_message_set_property (GObject *object, guint prop_id,
 			   const GValue *value, GParamSpec *pspec)
 {
 	SoupMessage *msg = SOUP_MESSAGE (object);
-	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
 	switch (prop_id) {
 	case PROP_METHOD:
-		priv->method = g_intern_string (g_value_get_string (value));
+                soup_message_set_method (msg, g_value_get_string (value));
 		break;
 	case PROP_URI:
 		soup_message_set_uri (msg, g_value_get_boxed (value));
@@ -214,10 +213,10 @@ soup_message_set_property (GObject *object, guint prop_id,
 		soup_message_set_first_party (msg, g_value_get_boxed (value));
 		break;
 	case PROP_PRIORITY:
-		priv->priority = g_value_get_enum (value);
+                soup_message_set_priority (msg, g_value_get_enum (value));
 		break;
-	case PROP_OPTIONS_PING:
-		priv->options_ping = g_value_get_boolean (value);
+	case PROP_IS_OPTIONS_PING:
+                soup_message_set_is_options_ping (msg, g_value_get_boolean (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -275,8 +274,8 @@ soup_message_get_property (GObject *object, guint prop_id,
 	case PROP_PRIORITY:
 		g_value_set_enum (value, priv->priority);
 		break;
-	case PROP_OPTIONS_PING:
-                g_value_set_boolean (value, priv->options_ping);
+	case PROP_IS_OPTIONS_PING:
+                g_value_set_boolean (value, priv->is_options_ping);
                 break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -754,16 +753,17 @@ soup_message_class_init (SoupMessageClass *message_class)
 				   G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * SoupMessage:options-ping:
+	 * SoupMessage:is-options-ping:
 	 *
 	 * The #SoupMessage is intended to be used to send
-         * `OPTIONS *` to a server and the path of
-         * #SoupMessage:uri will be ignored.
+         * `OPTIONS *` to a server. When set to %TRUE, the
+         * path of #SoupMessage:uri will be ignored and
+         * #SoupMessage:method set to %SOUP_METHOD_OPTIONS.
 	 */
 	g_object_class_install_property (
-		object_class, PROP_OPTIONS_PING,
-		g_param_spec_boolean ("options-ping",
-				      "Options Ping",
+		object_class, PROP_IS_OPTIONS_PING,
+		g_param_spec_boolean ("is-options-ping",
+				      "Is Options Ping",
 				      "The message is an OPTIONS ping",
                                       FALSE,
 				      G_PARAM_READWRITE |
@@ -778,7 +778,7 @@ soup_message_class_init (SoupMessageClass *message_class)
  * 
  * Creates a new empty #SoupMessage, which will connect to @uri
  *
- * Returns: (nullable): the new #SoupMessage (or %NULL if @uri
+ * Returns: (transfer full) (nullable): the new #SoupMessage (or %NULL if @uri
  * could not be parsed).
  */
 SoupMessage *
@@ -810,17 +810,41 @@ soup_message_new (const char *method, const char *uri_string)
  * 
  * Creates a new empty #SoupMessage, which will connect to @uri
  *
- * Returns: the new #SoupMessage
+ * Returns: (transfer full): the new #SoupMessage
  */
 SoupMessage *
 soup_message_new_from_uri (const char *method, GUri *uri)
 {
+        g_return_val_if_fail (method != NULL, NULL);
+        g_return_val_if_fail (SOUP_URI_IS_VALID (uri), NULL);
+
 	return g_object_new (SOUP_TYPE_MESSAGE,
 			     "method", method,
 			     "uri", uri,
 			     NULL);
 }
 
+/**
+ * soup_message_new_options_ping:
+ * @base_uri: the destination endpoint (as a #GUri)
+ *
+ * Creates a new #SoupMessage to send `OPTIONS *` to a server. The path of @base_uri
+ * will be ignored.
+ *
+ * Returns: (transfer full): the new #SoupMessage
+ */
+SoupMessage *
+soup_message_new_options_ping (GUri *base_uri)
+{
+        g_return_val_if_fail (SOUP_URI_IS_VALID (base_uri), NULL);
+
+        return g_object_new (SOUP_TYPE_MESSAGE,
+                             "method", SOUP_METHOD_OPTIONS,
+                             "uri", base_uri,
+                             "is-options-ping", TRUE,
+                             NULL);
+}
+
 /**
  * soup_message_new_from_encoded_form:
  * @method: the HTTP method for the created request (GET, POST or PUT)
@@ -920,7 +944,7 @@ soup_message_new_from_multipart (const char    *uri_string,
  * soup_message_set_request_body:
  * @msg: the message
  * @content_type: (allow-none): MIME Content-Type of the body, or %NULL if unknown
- * @stream: (allow-none): a #GPollableInputStream to read the request body from
+ * @stream: (allow-none): a #GInputStream to read the request body from
  * @content_length: the byte length of @stream or -1 if unknown
  *
  * Set the request body of a #SoupMessage.
@@ -928,7 +952,6 @@ soup_message_new_from_multipart (const char    *uri_string,
  * not be changed if present.
  * The request body needs to be set again in case @msg is restarted
  * (in case of redirection or authentication).
- * @stream must implement #GPollableInputStream.
  */
 void
 soup_message_set_request_body (SoupMessage  *msg,
@@ -937,7 +960,7 @@ soup_message_set_request_body (SoupMessage  *msg,
                                gssize        content_length)
 {
         g_return_if_fail (SOUP_IS_MESSAGE (msg));
-        g_return_if_fail (stream == NULL || G_IS_POLLABLE_INPUT_STREAM (stream));
+        g_return_if_fail (stream == NULL || G_IS_INPUT_STREAM (stream));
         g_return_if_fail (content_length == -1 || content_length >= 0);
 
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
@@ -1264,6 +1287,23 @@ soup_message_get_uri_for_auth (SoupMessage *msg)
 	return priv->uri;
 }
 
+static void
+soup_message_set_tls_certificate (SoupMessage         *msg,
+                                  GTlsCertificate     *tls_certificate,
+                                  GTlsCertificateFlags tls_errors)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        if (priv->tls_certificate == tls_certificate && priv->tls_certificate_errors == tls_errors)
+                return;
+
+        g_clear_object (&priv->tls_certificate);
+        priv->tls_certificate = tls_certificate ? g_object_ref (tls_certificate) : NULL;
+        priv->tls_certificate_errors = tls_errors;
+        g_object_notify (G_OBJECT (msg), "tls-certificate");
+        g_object_notify (G_OBJECT (msg), "tls-certificate-errors");
+}
+
 SoupConnection *
 soup_message_get_connection (SoupMessage *msg)
 {
@@ -1298,20 +1338,9 @@ re_emit_tls_certificate_changed (SoupMessage    *msg,
 				 GParamSpec     *pspec,
 				 SoupConnection *conn)
 {
-	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
-	GTlsCertificate *tls_certificate;
-	GTlsCertificateFlags tls_errors;
-
-	tls_certificate = soup_connection_get_tls_certificate (conn);
-	tls_errors = soup_connection_get_tls_certificate_errors (conn);
-	if (priv->tls_certificate == tls_certificate && priv->tls_certificate_errors == tls_errors)
-		return;
-
-	g_clear_object (&priv->tls_certificate);
-	priv->tls_certificate = tls_certificate ? g_object_ref (tls_certificate) : NULL;
-	priv->tls_certificate_errors = tls_errors;
-	g_object_notify (G_OBJECT (msg), "tls-certificate");
-	g_object_notify (G_OBJECT (msg), "tls-certificate-errors");
+        soup_message_set_tls_certificate (msg,
+                                          soup_connection_get_tls_certificate (conn),
+                                          soup_connection_get_tls_certificate_errors (conn));
 }
 
 void
@@ -1320,6 +1349,9 @@ soup_message_set_connection (SoupMessage    *msg,
 {
 	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
+        if (priv->connection == conn)
+                return;
+
 	if (priv->connection) {
 		g_signal_handlers_disconnect_by_data (priv->connection, msg);
 		g_object_remove_weak_pointer (G_OBJECT (priv->connection), (gpointer*)&priv->connection);
@@ -1330,7 +1362,9 @@ soup_message_set_connection (SoupMessage    *msg,
 		return;
 
 	g_object_add_weak_pointer (G_OBJECT (priv->connection), (gpointer*)&priv->connection);
-	re_emit_tls_certificate_changed (msg, NULL, conn);
+        soup_message_set_tls_certificate (msg,
+                                          soup_connection_get_tls_certificate (priv->connection),
+                                          soup_connection_get_tls_certificate_errors (priv->connection));
 
 	g_signal_connect_object (priv->connection, "event",
 				 G_CALLBACK (re_emit_connection_event),
@@ -1356,20 +1390,17 @@ soup_message_cleanup_response (SoupMessage *msg)
 {
 	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
+        g_object_freeze_notify (G_OBJECT (msg));
+
 	soup_message_headers_clear (priv->response_headers);
 
-	priv->status_code = SOUP_STATUS_NONE;
-	if (priv->reason_phrase) {
-		g_free (priv->reason_phrase);
-		priv->reason_phrase = NULL;
-	}
-	priv->http_version = priv->orig_http_version;
+        soup_message_set_status (msg, SOUP_STATUS_NONE, NULL);
+        soup_message_set_http_version (msg, priv->orig_http_version);
 
-	g_object_notify (G_OBJECT (msg), "status-code");
-	g_object_notify (G_OBJECT (msg), "reason-phrase");
-	g_object_notify (G_OBJECT (msg), "http-version");
-	g_object_notify (G_OBJECT (msg), "tls-certificate");
-	g_object_notify (G_OBJECT (msg), "tls-certificate-errors");
+        if (!priv->connection)
+                soup_message_set_tls_certificate (msg, NULL, 0);
+
+        g_object_thaw_notify (G_OBJECT (msg));
 }
 
 /**
@@ -1410,6 +1441,7 @@ soup_message_set_flags (SoupMessage *msg, SoupMessageFlags flags)
 	SoupMessagePrivate *priv;
 
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+
 	priv = soup_message_get_instance_private (msg);
 
 	if (priv->msg_flags == flags)
@@ -1476,7 +1508,7 @@ soup_message_query_flags (SoupMessage     *msg,
         g_return_val_if_fail (SOUP_IS_MESSAGE (msg), FALSE);
 
         priv = soup_message_get_instance_private (msg);
-	return priv->msg_flags & flags;
+	return !!(priv->msg_flags & flags);
 }
 
 /**
@@ -1510,10 +1542,10 @@ soup_message_remove_flags (SoupMessage     *msg,
 void
 soup_message_set_http_version (SoupMessage *msg, SoupHTTPVersion version)
 {
-	SoupMessagePrivate *priv;
+	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-	g_return_if_fail (SOUP_IS_MESSAGE (msg));
-	priv = soup_message_get_instance_private (msg);
+        if (priv->http_version == version)
+                return;
 
 	priv->http_version = version;
 	if (priv->status_code == SOUP_STATUS_NONE)
@@ -1601,15 +1633,27 @@ void
 soup_message_set_uri (SoupMessage *msg, GUri *uri)
 {
 	SoupMessagePrivate *priv;
+        GUri *normalized_uri;
 
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
         g_return_if_fail (SOUP_URI_IS_VALID (uri));
+
 	priv = soup_message_get_instance_private (msg);
 
-	if (priv->uri)
-		g_uri_unref (priv->uri);
-	priv->uri = soup_uri_copy_with_normalized_flags (uri);
+        normalized_uri = soup_uri_copy_with_normalized_flags (uri);
+        if (!normalized_uri)
+                return;
 
+        if (priv->uri) {
+                if (soup_uri_equal (priv->uri, normalized_uri)) {
+                        g_uri_unref (normalized_uri);
+                        return;
+                }
+
+                g_uri_unref (priv->uri);
+        }
+
+	priv->uri = normalized_uri;
 	g_object_notify (G_OBJECT (msg), "uri");
 }
 
@@ -1648,12 +1692,22 @@ soup_message_set_status (SoupMessage *msg,
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-	g_free (priv->reason_phrase);
+        g_object_freeze_notify (G_OBJECT (msg));
 
-	priv->status_code = status_code;
-	priv->reason_phrase = g_strdup (reason_phrase ? reason_phrase : soup_status_get_phrase (status_code));
-	g_object_notify (G_OBJECT (msg), "status-code");
-	g_object_notify (G_OBJECT (msg), "reason-phrase");
+        if (priv->status_code != status_code) {
+                priv->status_code = status_code;
+                g_object_notify (G_OBJECT (msg), "status-code");
+        }
+
+        if (reason_phrase) {
+                soup_message_set_reason_phrase (msg, reason_phrase);
+        } else {
+                soup_message_set_reason_phrase (msg, priv->status_code != SOUP_STATUS_NONE ?
+                                                soup_status_get_phrase (priv->status_code) :
+                                                NULL);
+        }
+
+        g_object_thaw_notify (G_OBJECT (msg));
 }
 
 /**
@@ -1794,6 +1848,8 @@ soup_message_set_first_party (SoupMessage *msg,
 
 	priv = soup_message_get_instance_private (msg);
         first_party_normalized = soup_uri_copy_with_normalized_flags (first_party);
+        if (!first_party_normalized)
+                return;
 
 	if (priv->first_party) {
 		if (soup_uri_equal (priv->first_party, first_party_normalized)) {
@@ -1855,6 +1911,8 @@ soup_message_set_site_for_cookies (SoupMessage *msg,
 
 	priv = soup_message_get_instance_private (msg);
         site_for_cookies_normalized = soup_uri_copy_with_normalized_flags (site_for_cookies);
+        if (!site_for_cookies_normalized)
+                return;
 
 	if (priv->site_for_cookies) {
 		if (soup_uri_equal (priv->site_for_cookies, site_for_cookies_normalized)) {
@@ -2001,9 +2059,16 @@ void
 soup_message_set_priority (SoupMessage        *msg,
 			   SoupMessagePriority priority)
 {
+        SoupMessagePrivate *priv;
+
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
 
-	g_object_set (msg, "priority", priority, NULL);
+        priv = soup_message_get_instance_private (msg);
+        if (priv->priority == priority)
+                return;
+
+        priv->priority = priority;
+	g_object_notify (G_OBJECT (msg), "priority");
 }
 
 /**
@@ -2176,8 +2241,12 @@ soup_message_set_reason_phrase (SoupMessage *msg, const char *reason_phrase)
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
+        if (g_strcmp0 (priv->reason_phrase, reason_phrase) == 0)
+                return;
+
         g_free (priv->reason_phrase);
         priv->reason_phrase = g_strdup (reason_phrase);
+        g_object_notify (G_OBJECT (msg), "reason-phrase");
 }
 
 void
@@ -2185,16 +2254,58 @@ soup_message_set_method (SoupMessage *msg,
                          const char  *method)
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+        const char *new_method = g_intern_string (method);
 
-        g_return_if_fail (method != NULL);
+        if (priv->method == new_method)
+                return;
 
-        priv->method = g_intern_string (method);
+        priv->method = new_method;
+        g_object_notify (G_OBJECT (msg), "method");
 }
 
+/**
+ * soup_message_get_is_options_ping:
+ * @msg: a #SoupMessage
+ *
+ * Gets whether @msg is intended to be used to send `OPTIONS *` to a server.
+ *
+ * Returns: %TRUE if the message is options ping, or %FALSE otherwise
+ */
 gboolean
-soup_message_is_options_ping (SoupMessage *msg)
+soup_message_get_is_options_ping (SoupMessage *msg)
 {
-        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+        SoupMessagePrivate *priv;
 
-        return priv->options_ping;
+        g_return_val_if_fail (SOUP_IS_MESSAGE (msg), FALSE);
+
+        priv = soup_message_get_instance_private (msg);
+
+        return priv->is_options_ping;
+}
+
+/**
+ * soup_message_set_is_options_ping:
+ * @msg: a #SoupMessage
+ * @is_options_ping: the value to set
+ *
+ * Set whether @msg is intended to be used to send `OPTIONS *` to a server.
+ * When set to %TRUE, the path of #SoupMessage:uri will be ignored and
+ * #SoupMessage:method set to %SOUP_METHOD_OPTIONS.
+ */
+void
+soup_message_set_is_options_ping (SoupMessage *msg,
+                                  gboolean     is_options_ping)
+{
+        SoupMessagePrivate *priv;
+
+        g_return_if_fail (SOUP_IS_MESSAGE (msg));
+
+        priv = soup_message_get_instance_private (msg);
+        if (priv->is_options_ping == is_options_ping)
+                return;
+
+        priv->is_options_ping = is_options_ping;
+        g_object_notify (G_OBJECT (msg), "is-options-ping");
+        if (priv->is_options_ping)
+                soup_message_set_method (msg, SOUP_METHOD_OPTIONS);
 }
