@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2000-2003, Ximian, Inc.
  */
@@ -24,6 +24,9 @@ SOUP_AVAILABLE_IN_ALL
 SoupMessage   *soup_message_new_from_uri          (const char        *method,
 						   GUri              *uri);
 
+SOUP_AVAILABLE_IN_ALL
+SoupMessage   *soup_message_new_options_ping      (GUri              *base_uri);
+
 SOUP_AVAILABLE_IN_ALL
 SoupMessage   *soup_message_new_from_encoded_form (const char        *method,
 						   const char        *uri_string,
@@ -44,9 +47,6 @@ void           soup_message_set_request_body_from_bytes (SoupMessage  *msg,
 							 GBytes       *bytes);
 
 SOUP_AVAILABLE_IN_ALL
-void             soup_message_set_http_version    (SoupMessage       *msg,
-						   SoupHTTPVersion    version);
-SOUP_AVAILABLE_IN_ALL
 SoupHTTPVersion  soup_message_get_http_version    (SoupMessage       *msg);
 
 SOUP_AVAILABLE_IN_ALL
@@ -162,4 +162,11 @@ SoupMessageHeaders *soup_message_get_request_headers  (SoupMessage  *msg);
 SOUP_AVAILABLE_IN_ALL
 SoupMessageHeaders *soup_message_get_response_headers (SoupMessage  *msg);
 
+SOUP_AVAILABLE_IN_ALL
+gboolean            soup_message_get_is_options_ping  (SoupMessage  *msg);
+
+SOUP_AVAILABLE_IN_ALL
+void                soup_message_set_is_options_ping  (SoupMessage  *msg,
+                                                       gboolean      is_options_ping);
+
 G_END_DECLS
